/* -*- mode: c++; c-basic-offset: 4; indent-tabs-mode: nil -*-

this file is part of rcssserver3D
Fri May 9 2003
Copyright (C) 2002,2003 Koblenz University
Copyright (C) 2003 RoboCup Soccer Server 3D Maintenance Group
$Id: kicknrun.cpp,v 1.4 2004/07/01 18:31:48 kubo Exp $

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "kicknrun.h"
#include <sstream>

using namespace zeitgeist;
using namespace std;
using namespace boost;
using namespace oxygen;
using namespace salt;

KickNRun::KickNRun() : Soccer()
{
}


KickNRun::~KickNRun()
{
}


void KickNRun::Behave()
{
    TTeamIndex ti = mWM->GetMyTeam();
    if (ti != TI_LEFT && ti != TI_RIGHT) {
        GetLog()->Debug() << "Init failed:"
                          << mWM->GetTeamUnum()
                          << endl;
        //InitAgent();
        return;
    }
    TPlayMode myKickOff = (ti == TI_LEFT)? PM_KickOff_Left :PM_KickOff_Right;
    TPlayMode myKickIn  = (ti == TI_LEFT)? PM_KickIn_Left  :PM_KickIn_Right;
    TPlayMode pm = mWM->GetPlayMode();
    switch (pm)
        {
        case PM_BeforeKickOff :
            BehaveBeforeKickOff();
            break;

        case PM_KickOff_Left :
        case PM_KickOff_Right :
            if (pm = myKickOff){
                BehaveMyKickOff();
            } else {
                BehaveTheirKickOff();
            }
            break;

        case PM_PlayOn :
            BehavePlayOn();
            break;

        case PM_KickIn_Left :
        case PM_KickIn_Right :
            if (pm = myKickIn){
                BehaveMyKickIn();
            } else {
                BehaveTheirKickIn();
            }
            break;

        default:
            GetLog()->Debug() << "Unknown PM: " << pm << endl;
            BehavePlayOn();
            break;
        }

}


void KickNRun::BehaveBeforeKickOff()
{
    Beam(mWM->formations[WorldModel::FT_initial].player[mWM->GetTeamUnum()].pos);
    Drive(Vector3f(0, 0, 0));
}


void KickNRun::BehaveMyKickOff()
{
    if (mWM->GetTeamUnum() == 9) {
        BehavePlayOn();
    }
}

void KickNRun::BehaveTheirKickOff()
{
    // do nothing
    Drive(Vector3f(0,0,0));
}

void KickNRun::BehaveMyKickIn()
{
    // for now just kick and run
    BehavePlayOn(true);
}

void KickNRun::BehaveTheirKickIn()
{
    // do nothing
    Drive(Vector3f(0,0,0));
}

void KickNRun::BehavePlayOn(bool kickin)
{
    //if (mWM->GetTeamUnum() == 1) {
     //   BehavePlayOn_GK();
      //  return;
    //}
    Vector3f g[3];
    g[0] = Vector3f(mWM->GetFieldLength() / 2, 3, 0);
    g[1] = Vector3f(mWM->GetFieldLength() / 2, -3, 0);
    g[2] = Vector3f(mWM->GetFieldLength() / 2, -3, 0);
    Vector3f myPos = mWM->GetMyPosition();

#if 0
    Vector3f goalPos = (((g[0]-myPos).Length() < (g[1]-myPos).Length())?
                        g[0]:
                        g[1]);
#else
    Vector3f goalPos = g[rand()%3];
#endif

    if (kickin) {
        goalPos = Vector3f(0, 0, 0);
    }



    float minKickDist = mWM->GetMinimalKickDistance();
    WorldModel::VisionObject myVO = mWM->GetMyObject();
    Vector3f myV = mWM->GetObjectSpeed(myVO);

    Vector3f ballPos = mWM->GetObjectPosition(WorldModel::VO_BALL);
    GetLog()->Debug()
        << "Ball:\t" <<  ballPos.x() << "\t" << ballPos.y() << endl;
    Vector3f ballv = mWM->GetObjectSpeed(WorldModel::VO_BALL);
    ballPos = ballPos + ballv * mWM->GetNeedStepsSetTo(myVO, WorldModel::VO_BALL);
    GetLog()->Debug()
        << "Ball2:\t" <<  ballPos.x() << "\t" << ballPos.y() << endl;

    Vector3f ball2goal = goalPos - ballPos;
    Vector3f kickPos;

    if (ball2goal.Length() < 15) {
        kickPos = ballPos - (ball2goal.Normalized() * minKickDist);
    } else {
        for (int i = 1; i <= 11; i++) {
            if ( i != mWM->GetTeamUnum() ) {
                WorldModel::VisionObject vo = static_cast<WorldModel::VisionObject>(static_cast<int>(WorldModel::VO_OUR0) + i);
                Vector3f passPos = mWM->GetObjectPosition(vo);
                if ((passPos.x() - myPos.x()) > 5){
                    double l = (passPos - myPos).Length();
                    if (10 < l && l < 20) {
                        passPos += Vector3f(5, 0, 0);
                        kickPos = ballPos - ((passPos - ballPos).Normalized() * minKickDist);
                        goto out;
                    }
                }
            }
        }
        kickPos = ballPos - (ball2goal.Normalized() * minKickDist);
    }
 out:
    Vector3f dl2KickPos = kickPos - myPos;

    if (dl2KickPos.Length() < minKickDist ) {
        if (myPos.x() < 0) {
            Kick(30, 100);
        } else if (myPos.x() < 25) {
            Kick(20, 100);
        } else {
            Kick(10, 100);
        }
    } else if (mWM->GetFastestInSetTo(WorldModel::SET_TEAMMATE_NOGK, WorldModel::VO_BALL) == myVO) {
        if (dl2KickPos.Length() > 0.5 ) {
            Drive((kickPos - myPos).Normalized() * 100);
        } else if (dl2KickPos.Length() > 0.35 && myV.Length() > 0.2) {
            Drive(myV.Normalized() * -80); // break.
        } else if (dl2KickPos.Length() > minKickDist ) {
            Drive((kickPos - myPos).Normalized() * 20);
        } else {
            Drive((kickPos - myPos).Normalized() * 20);
        }
    } else if((mWM->GetStrategicPosition() - myPos).Length() > 1.5 + fabs(myPos.x() - ballPos.x()) / 10.0) {
        Drive((mWM->GetStrategicPosition() - myPos).Normalized() * 30);
    }
}


void KickNRun::BehavePlayOn_GK()
{
    Vector3f goalPos = Vector3f(mWM->GetFieldLength() / 2, 0, 0);

    Vector3f myPos = mWM->GetMyPosition();
    float minKickDist = mWM->GetMinimalKickDistance();
    WorldModel::VisionObject myVO = mWM->GetMyObject();
    Vector3f myV = mWM->GetObjectSpeed(myVO);

    Vector3f ballPos = mWM->GetObjectPosition(WorldModel::VO_BALL);
    GetLog()->Debug()
        << "Ball:\t" <<  ballPos.x() << "\t" << ballPos.y() << endl;
    Vector3f ballv = mWM->GetObjectSpeed(WorldModel::VO_BALL);
    ballPos = ballPos + ballv * mWM->GetNeedStepsSetTo(myVO, WorldModel::VO_BALL);
    GetLog()->Debug()
        << "Ball2:\t" <<  ballPos.x() << "\t" << ballPos.y() << endl;

    Vector3f ball2goal = goalPos - ballPos;
    Vector3f kickPos;

    if (ball2goal.Length() < 15) {
        kickPos = ballPos - (ball2goal.Normalized() * minKickDist);
    } else {
        for (int i = 1; i <= 11; i++) {
            if ( i != mWM->GetTeamUnum() ) {
                WorldModel::VisionObject vo = static_cast<WorldModel::VisionObject>(static_cast<int>(WorldModel::VO_OUR0) + i);
                Vector3f passPos = mWM->GetObjectPosition(vo);
                if ((passPos.x() - myPos.x()) > 5){
                    double l = (passPos - myPos).Length();
                    if (10 < l && l < 20) {
                        passPos += Vector3f(5, 0, 0);
                        kickPos = ballPos - ((passPos - ballPos).Normalized() * minKickDist);
                        goto out;
                    }
                }
            }
        }
        kickPos = ballPos - (ball2goal.Normalized() * minKickDist);
    }
 out:
    Vector3f dl2KickPos = kickPos - myPos;

    if (dl2KickPos.Length() < minKickDist ) {
        Kick(20, 100);
    } else if (mWM->GetFastestInSetTo(WorldModel::SET_TEAMMATE, WorldModel::VO_BALL) == myVO) {
        if (ballPos.x() < -30 && abs(ballPos.y()) < 10) { 
            if (dl2KickPos.Length() > 0.5 ) {
                Drive((kickPos - myPos).Normalized() * 100);
            } else if (dl2KickPos.Length() > 0.35 && myV.Length() > 0.2) {
                Drive(myV.Normalized() * -80); // break.
            } else if (dl2KickPos.Length() > minKickDist ) {
                Drive((kickPos - myPos).Normalized() * 20);
            } else {
                Drive((kickPos - myPos).Normalized() * 20);
            }
        }
    } else {
        Vector3f basePos = Vector3f((-mWM->GetFieldLength() / 2) + 1, 0, 0);
        Vector3f ballPos2 = mWM->GetObjectPosition(WorldModel::VO_BALL);
        double x = (ballPos2 - basePos).x() / 15;
        double y = (ballPos2 - basePos).y() / 3;
        double minx = - mWM->GetFieldLength() / 2;
        double maxx = minx + 15;
        double miny = - mWM->GetGoalWidth()/ 2 - 2;
        double maxy = - miny;

        x = (x > minx)? x: minx;
        x = (x < maxx)? x: maxx;
        y = (y > miny)? y: miny;
        y = (y < maxy)? y: maxy;

        Vector3f strPos = basePos + Vector3f(x, y, 0);
        if((strPos - myPos).Length() > 1.5 + fabs(myPos.x() - ballPos.x()) / 10.0) {
            Drive((strPos - myPos).Normalized() * 100);
        }
    }
}

